/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sys.testsys.modulApplUeb;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import de.bsvrz.sys.testsys.lib.MyLogger;

/**
 * Behälterklasse ; Datenstrukturen zur Aufnahme der Start/Stopp- und der
 * Applikationsdaten.
 *
 * Klasse wird als Singleton realisiert.
 *
 * Aenderung am 17.06.09: m_applDatenListe synchronisiert
 *
 * @author ChriestenJ
 */
public class ApplUebDaten
{
  /**
   * Die einzige Instanz dieser Klasse
   */
  private static ApplUebDaten m_prozessDaten = null;

  /**
   * singleton Konstruktor
   */
  private ApplUebDaten()
  {
    // m_applDatenListe = new HashMap<String, ApplUebDaten.ApplDaten>();
    // m_startStoppDatenListe = new HashMap<String, StartStoppDaten>();
  }

  /**
   * liefert singleton Instanz der Klasse
   * @return Instanz der Klasse
   */
  public static synchronized ApplUebDaten getInstance()
  {
    if ( m_prozessDaten == null )
    {
      m_prozessDaten = new ApplUebDaten();
    }
    return m_prozessDaten;
   }

  // ---------------------------------------------------------------------------
  // Membervariablen
  // ---------------------------------------------------------------------------
  // HashMap
  // Key = Inkarnation == ProzessInfo

  /**
   * aktuelle Info von Start/Stopp
   * Key = ProzessInfo (==Inkarnation)
   */
    //private HashMap<String, StartStoppDaten> m_startStoppDatenListe = new HashMap<String, StartStoppDaten>();
    private Map<String, StartStoppDaten> m_startStoppDatenListe =  Collections.synchronizedMap(new HashMap<String, StartStoppDaten>());

    //private HashMap<String, StartStoppDaten> m_startStoppDatenListe = (HashMap<String, StartStoppDaten>) Collections.synchronizedMap(new HashMap<String, StartStoppDaten>());


  /**
   * aktuelle Info der Applikation
   * Key = Objekt Id der Applikation
   * map wird jetzt synchronisiert. JC 15.06.2009.
   */
  //private HashMap<String, ApplUebDaten.ApplDaten> m_applDatenListe = new HashMap<String, ApplUebDaten.ApplDaten>();
  private Map<String, ApplUebDaten.ApplDaten> m_applDatenListe =  Collections.synchronizedMap(new HashMap<String, ApplUebDaten.ApplDaten>());

  /**
   * aktuelle Info von Start/Stopp
   * Key = ProzessInfo (==Inkarnation)
   * @return Liste der aktuellen ProzessInfo Daten
   */
  public Map<String, StartStoppDaten> get_startStoppDatenListe()
  {
    return m_startStoppDatenListe;
  }

  /**
   * Liefert Liste der aktuellen Applikationsdaten
   * Key = ProzessInfo (==Inkarnation)
   * @return Liste der aktuellen Applikationsdaten
   */
  public Map<String, ApplUebDaten.ApplDaten> get_applDatenListe()
  {
    return m_applDatenListe;
  }

  /**
   * Liefert Behälter zur Aufnahme der Start/Stopp Information
   * @return Behälterinstanz
   */
  public StartStoppDaten get_startStoppDatenInstanz()
  {
    StartStoppDaten ss = new StartStoppDaten();
    return(ss);
  }

  /**
   * Liefert Behälter Aufnahme der angemeldete Appliaktionen
   * @return Behälterinstanz
   */
  public ApplDaten get_applDatenInstanz()
  {
    ApplDaten ad = new ApplDaten();
    return(ad);
  }

  /**
   * Hilfsklasse; definiert die Datentruktur zur Aufnahme der
   * Start/Stopp Daten
   *
   * @author ChriestenJ
   */
	public class StartStoppDaten implements ApplUebDaten.IAus
	{
	  /**
	   * Start/Stopp ProzessId
	   * == Appl/Inkarnation
	   */
	  public String m_prozessId;

    /**
     * Pid des Rechners
     */
    public String m_rechnerPid = null;

	  /**
	   * Start/Stopp Name
	   * == Appl/Inkarnation
	   */
	  public String m_ssName;

	  /**
	   * Start/Stopp Datei
	   * == Appl/Inkarnation
	   */
	  public String m_ssDatei;

	  /**
	   * Start/Stopp, aktueller Zustand
	   */
	  public String m_ssZustand = null;

    /**
     * Start/Stopp, vorhergehender Zustand
     */
    public String m_ssZustand_vorher = null;

	  /**
	   * Start/Stopp, Startzeitpunkt
	   */
	  public String m_ssStartZP;

	  /**
	   * Start/Stopp, InitialisierungsZeitpunkt
	   */
	  public String m_ssInitZP;

    /**
     * Start/Stopp, nächster Startzeitpunkt
     */
    public String m_ssNaechstStartZP;

    /**
     * temporäres Indikatorfeld
     * Zustand geaendert seit (ms seit 1900)
     */
    public Long m_ssZustandAendZeit = null;

    /**
     * temporäres Indikatorfeld
     * Löschzeit (ms seit 1900)
     */
    public Long m_loeZeit = null;

    /**
     * temporäres Indiaktorfeld
     * 0: loeschen
     * 1: alt
     * 2: neu
     */
    public Integer m_tmpNeuzust = null;

    /**
     * Ergebnis der letzten Konsistenzprüfung dieses Datensatzes<br>
     * 0: keine Prüfung, 1: ok, 3: Fehler
     */
    public int m_pruef_erg = 0;

    /**
     * SS Zustand bei der letzen letzten Konsistenzprüfung
     */
    public String m_pruef_ssZustand = null;

    /**
     * Liste der Aufrufparameter
     * Es werden nur die Parameter ohne startendem - gespeichert
     * == Aufruf des Programms
     */
    public List<String> m_aufrufParameter = new ArrayList<String>();

    /**
     * Arbeitsflag. Muss der Eintrag geprüft werden?
     */
    public Boolean m_nichtPruefen = null;


    /**
	   * Ausgabefunktion. Schreibt die aktuellen Objektdaten auf die Konsole.
	   *
	   */
	  public void aus()
	  {
      Long lokzustAendZeit = m_ssZustandAendZeit==null ? null : (System.currentTimeMillis()-m_ssZustandAendZeit)/1000;
      Long lokloeZeit = m_loeZeit==null ? null : (System.currentTimeMillis()-m_loeZeit)/1000;
  		MyLogger.logln("TTT", "m_prozessId=" + m_prozessId
          + " ,ssZust=" + m_ssZustand
          + " ,prZust=" + m_pruef_ssZustand
          + " ,prErg=" + m_pruef_erg
          + " ,rechPid=" + m_rechnerPid
          + " ,ssZust_vorher=" + m_ssZustand_vorher
          + " ,zustAendZeit=" + lokzustAendZeit
          + " ,LoeZeit=" + lokloeZeit
  	    + " ,ssname=" + m_ssName
  			+ " ,datei=" + m_ssDatei + " ,ssZustand=" + m_ssZustand
  			+ " ,startZp=" + m_ssStartZP + " ,iniZP="+m_ssInitZP
        + " ,exe="+ ((m_aufrufParameter.size() <= 0) ? null :m_aufrufParameter.get(0))
        + " ,nicht_prüfen?="+ m_nichtPruefen
        );
	  }
	}


  /**
   * setzt m_neuzust aller Datensaezte in der Applikationsdatenliste auf 0
   */
  public void setzeNeuzustApplDat0()
  {
    Iterator<Map.Entry<String, ApplUebDaten.ApplDaten>> it = m_applDatenListe.entrySet().iterator();
    synchronized ( m_applDatenListe )
    {
      while (it.hasNext())
      {
        Map.Entry<String, ApplUebDaten.ApplDaten> me = (Map.Entry<String, ApplUebDaten.ApplDaten>) it.next();
        ApplUebDaten.ApplDaten adat = me.getValue();
        adat.m_tmpNeuzust = 0;
      }
    }
  }

  /**
   * setzt m_neuzust aller Datensaezte in der StartStoppdatenliste auf 0
   */
  public void setzeNeuzustStartStoppDat0( )
  {
    synchronized(m_startStoppDatenListe)
    {
      Iterator<Map.Entry<String, ApplUebDaten.StartStoppDaten>> it = m_startStoppDatenListe.entrySet().iterator();
      while (it.hasNext())
      {
        Map.Entry<String, ApplUebDaten.StartStoppDaten> me = (Map.Entry<String, ApplUebDaten.StartStoppDaten>) it.next();
        ApplUebDaten.StartStoppDaten ssdat = me.getValue();
        ssdat.m_tmpNeuzust = 0;
      }
    }
  }


	/**
	 * Hilfsklasse; definiert die Datentruktur zur Aufnahme der
	 * Applikationsdaten
	 *
	 * @author ChriestenJ
	 */
	public class ApplDaten implements ApplUebDaten.IAus
	{
	  /**
	   * ApplikaitonsIs (=objId typ.applikation)
	   */
	  public Long m_applId = null;

	  /**
	   * Start/Stopp ProzessId
	   * == Appl/Inkarnation
	   */
	  public String m_prozessId = null;

    /**
     * Uebernahme seit
     * von der AngemeldeteAppl
     */
    public String m_seit;

    /**
     * Uebernahme benutzer
     * von der AngemeldeteAppl
     */
    public String m_benutzer;

    /**
     * Zustand, temporäres Indiaktorfeld
     * 0: loeschen
     * 1: alt
     * 2: neu
     */
    public Integer m_tmpNeuzust = null;

    /**
     * temporäres Indikatorfeld
     * Zeit neu (ms seit 1900)
     */
    public Long neuZeit = null;

    /**
     * temporäres Indikatorfeld
     * Löschzeit (ms seit 1900)
     */
    public Long loeZeit = null;

    /**
	   * Ist Applikaitons bereit initialisiert?
	   */
	  Boolean m_ist_initialisiert;

	  /**
	   * Ausgabefunktion. Schreibt die aktuellen Objektdaten auf die Konsole.
	   *
	   */
	  public void aus()
	  {
      Long lokloeZeit = loeZeit==null ? null : (System.currentTimeMillis()-loeZeit)/1000;
      Long lokneuZeit = neuZeit==null ? null : (System.currentTimeMillis()-neuZeit)/1000;
		MyLogger.logln("TTT", "m_applId=" + m_applId
			+ " ,inkarnation=" + m_prozessId
			+ " ,ist_initialisiert=" + m_ist_initialisiert
      +" ,neu seit"+lokneuZeit
      +" ,geloescht seit"+lokloeZeit
      + " ,neuzust="+m_tmpNeuzust
      + " ,ben="+m_benutzer
      + " ,seit="+m_seit
			);
	  }
	}

  // ---------------------------------------------------------------------------

	/**
	 * Interface, zur vereinheitlichung der Ausgabefunktionen alle HashMaps
	 * in dieser Klasse
	 */
	private interface IAus
	{
	  public void aus( );
	}

	/**
	 * gibt den Inhalt von ausMap zeilenweise aus.
	 * @param ausMap auszugebende Map
	 * @param titel Titel für die Ausgabe
	 */
  public void ausMap( Map<String, ?> ausMap, String titel)
  {
    if (ausMap == null)
      return;

    MyLogger.logln("TTT", "------------"+titel+"("+ausMap.size()+")"+"-------------------------------");
    Set<String> keys = ausMap.keySet();

    synchronized(ausMap)
    {
      for (Iterator<String> iterator = keys.iterator(); iterator.hasNext();)
      {
        String mykey = iterator.next();
        //MyLogger.log("TTT", "key=" + mykey + " ");
        Object object = ausMap.get( mykey );
        //if (object ins )
        if ( object instanceof IAus )
        {
          IAus iaus = (IAus) object;
          iaus.aus();
        }
        else
        {
          // Fehler
          System.err.println(this.getClass().getName()+".ausMap: Typ der Listenelemente nicht ausgabefähig.");
        }
      }
    }
    MyLogger.logln("TTT", "-----------------------------------------------------");
  }

}
