/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sys.testsys.modulStartStoppUeb;

import java.util.HashMap;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.sys.testsys.lib.ArgumentParser;
import de.bsvrz.sys.testsys.lib.MyLogger;
import de.bsvrz.sys.testsys.testsys.IBeendePlugin;
import de.bsvrz.sys.testsys.testsys.IGibZugriff;
import de.bsvrz.sys.testsys.testsys.IMeldungFehler;
import de.bsvrz.sys.testsys.testsys.davlib.DavLib;
import de.bsvrz.sys.testsys.testsys.davlib.IDaVInfoKonsument;
import de.bsvrz.sys.testsys.testsys.davlib.RechnerDaten;

/**
 * Überprüft Start/Stopp. Überwacht wird die Existenz des Senders der Attributgruppe rechner.ProzesInfo.zustand Der Sender ist Start/Stopp.
 *
 * @author Chriesten
 *
 */
public class ModulStartStoppUeb extends Thread implements IGibZugriff, IDaVInfoKonsument, IBeendePlugin
{
  /**
   * Aktuelle Verbindung zum Datenverteiler.
   */
  private ClientDavInterface m_davConnection;

  /**
   * Rueckrufobjekt zum Schreiben der Betriebsmeldunge
   */
  private IMeldungFehler m_MeldungFehler = null;

  /**
   * Hash Map mit allen Rechnerobjekten der aktuellen DAV Verbindung
   */
  private HashMap<String, RechnerDaten> m_davRechnerMap = new HashMap<String, RechnerDaten>();

  /**
   * Instanz der DavLib Hilfsklasse. Wird für DaV An- und Abmeldungen
   * verwendet.
   */
  private DavLib m_davLib;

  /**
   * rechner auf dem StartStopp überprüft wird. Sonderfälle: lokal, konfig = auf dem lokalen, auf allen im DAV konfigurierten null == lokal
   */
  private String m_rechner = null;

  // --------------------------------------
  // Konstruktoren
  // --------------------------------------

  /**
   * Parameterloser Konstruktor ohne Funkionalität ; los geht's in {@link ModulStartStoppUeb}{@link #steuereUeberwachung(IMeldungFehler, ClientDavInterface)}.
   */
  public ModulStartStoppUeb()
  {
    MyLogger.logln( "VERLAUF", "Konstruktur ohne Parameter " + this.getClass().getName() );
    initKonstruktor( "" );
  }

  /**
   * Konstruktor mit Argument String ; setzt Argumente ; richtig los geht's in {@link ModulStartStoppUeb}{@link #steuereUeberwachung(IMeldungFehler, ClientDavInterface)}.
   * Argumentepaare durch --- getrennt. Schlüssel-/Wertepaare durch = getrennt<br>
   *
   * @param argString optionaler Argumentstring "rechner=&lt;Wert&gt;" ; Wert in ("lokal", "konfig", &lt;DaV RechnerPid&gt;)
   *
   */
  public ModulStartStoppUeb( String argString )
  {
    MyLogger.logln( "VERLAUF", "Konstruktur mit string Parameter "
        + this.getClass().getName()
        + "\nargString: "
        + argString );
    initKonstruktor( argString );
  }

  /**
   * Hilfsmethode zur Vereinheitlichung der Konstruktoren. Funktionalität und Parameter siehe dort.
   *
   * @param argString
   *          siehe Konstruktor.
   */
  private void initKonstruktor( String argString )
  {
    // argString auswerten
    String meinargString = (argString == null) ? "" : argString;
    HashMap<String, String> schluesselWertPaare = ArgumentParser.parseArgString( meinargString,
        "---" );
    m_rechner = ArgumentParser.getWert( schluesselWertPaare, "rechner", false, false );
System.out.println("Rechner="+m_rechner);
  }

  // --------------------------------------
  // Initialisierung
  // --------------------------------------

  /**
   * initialisiert die plugin DAV Aktionen. Anmeldung der Rechner auf die Attributgruppe ProzessInfo. Abonnement der ProzessInfo Konsument
   */
  public void initDAV()
  {
    //
    // Notwendige Attributgruppen am DaV anmelden
    // rechner.ProzessInfo anmelden
    DavLib davlib = new DavLib( m_davConnection );
    m_davLib = davlib;

    // DaV Ergebnisdaten abonnieren
    davlib.abonniereDaVInfo( (IDaVInfoKonsument) this );

    // Attributgruppe rechner.prozesInfo anmelden
    davlib.anmeldeLeseProzessInfoList( m_davRechnerMap, m_rechner );
  }

  // -----------------------------------------------------------
  // Methoden der Schnittstelle IProzessInfoKonsument
  // -----------------------------------------------------------

  /**
   * Uberprüft die Existenz der Sender der Attributgruppe rechner.prozessInfo. Ein fehlender Sender wird als fehlende Start/Stopp Anwendung interpretiert. Eine
   * entsprechende Meldung wird ausgegeben. Siehe Schnittstellenbeschreibung.
   */
  public void neueDavInfo( ResultData dat )
  {
    // Verarbeite die Attribuggruppe atg.prozessInfo
    if ( !(dat.getDataDescription().getAttributeGroup().getPid().equals( "atg.prozessInfo" )) )
    {
      return;
    }

    // Absender Pid feststellen
    String rechnerPid = dat.getObject().getPid();

    // StartStopp ist Quelle für den Aspekt
    Boolean startStoppOk = dat.isSourceAvailable();

    RechnerDaten rd = m_davRechnerMap.get( rechnerPid );

    Integer ebene = new Integer( 1 ); // Fehlerebene, 1: ok, 3: Fehler
    String meldung = "Start/Stopp auf " + rd.m_name + "(" + rd.m_tcpip + ")";

    if ( startStoppOk )
    {
      ebene = Integer.valueOf( 1 );
      meldung = meldung + ": OK";
    }
    else
    {
      ebene = Integer.valueOf( 3 );
      meldung = meldung + " läuft nicht";
    }

    // Ergebnis letzte Pruefung
    Boolean pruef_erg_startStoppOK_letzt = rd.m_pruef_erg_startStoppOk;
    rd.m_pruef_erg_startStoppOk = startStoppOk;

    // Keine Änderung gegenüber der letzten Prüfung. Keine Meldung.
    if ( (pruef_erg_startStoppOK_letzt != null)
        && (rd.m_pruef_erg_startStoppOk == pruef_erg_startStoppOK_letzt) )
    {
      return;
    }

    m_MeldungFehler.fehlerGefunden( meldung, ebene, this.getClass().getName() );
  }

  // -------------------------------------------------------------------
  // Methoden der Schnittstelle GibZugriff
  // -------------------------------------------------------------------
  /**
   * siehe {@link IGibZugriff}{@link #steuereUeberwachung(IMeldungFehler, ClientDavInterface)}
   */
  public void steuereUeberwachung( IMeldungFehler pluginVerwalter, ClientDavInterface connection )
  {
    ClientDavInterface alte_conn = m_davConnection;
    m_MeldungFehler = pluginVerwalter;
    m_davConnection = connection;
    // DAV Connection notwendeig. Daher Initialisierung erst hier.
    if ( alte_conn == null )
      initDAV();
  }

  // -------------------------------------------------------------------
  // Methode der Schnittstelle IBeendePlugin
  // -------------------------------------------------------------------
  /**
   * Abmeldung aller vom Plugin angemeldeten Datenverteilerobjekte.
   * siehe {@link IBeendePlugin}{@link #beenden()}.
   */
  public void beenden()
  {
    MyLogger.logln( "TTT_BEENDEN", "Plugin" + this.getClass().getName() + " soll sich beenden." );
    
    m_davLib.abmeldeLeseProzessInfoAuto( m_davRechnerMap ); // rechner.ProzessInfo abmelden
  }
  // -------------------------------------------------------------------

}
